/*
 * Copyright (C) 2023 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "RemoteScrollbarsController.h"

#if PLATFORM(MAC)

#include <WebCore/ScrollableArea.h>
#include <WebCore/ScrollingCoordinator.h>

namespace WebKit {

RemoteScrollbarsController::RemoteScrollbarsController(WebCore::ScrollableArea& scrollableArea, WebCore::ScrollingCoordinator* coordinator)
    : ScrollbarsController(scrollableArea)
    , m_coordinator(ThreadSafeWeakPtr<WebCore::ScrollingCoordinator>(coordinator))
{
}

void RemoteScrollbarsController::mouseEnteredContentArea()
{
    if (auto scrollingCoordinator = m_coordinator.get())
        scrollingCoordinator->setMouseIsOverContentArea(scrollableArea(), true);
}

void RemoteScrollbarsController::mouseExitedContentArea()
{
    if (auto scrollingCoordinator = m_coordinator.get())
        scrollingCoordinator->setMouseIsOverContentArea(scrollableArea(), false);
}

void RemoteScrollbarsController::mouseMovedInContentArea()
{
    if (auto scrollingCoordinator = m_coordinator.get())
        scrollingCoordinator->setMouseMovedInContentArea(scrollableArea());
}

void RemoteScrollbarsController::mouseEnteredScrollbar(WebCore::Scrollbar* scrollbar) const
{
    if (auto scrollingCoordinator = m_coordinator.get())
        scrollingCoordinator->setMouseIsOverScrollbar(scrollbar, true);
}

void RemoteScrollbarsController::mouseExitedScrollbar(WebCore::Scrollbar* scrollbar) const
{
    if (auto scrollingCoordinator = m_coordinator.get())
        scrollingCoordinator->setMouseIsOverScrollbar(scrollbar, false);
}

}
#endif // PLATFORM(MAC)
